/* mswindows.c -- Various Windows functions.
   Copyright (C) 1995, 1996, 1997, 1998  Free Software Foundation, Inc.

This file is part of Wget.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <winsock.h>
#include <string.h>
#include <assert.h>

#include "wget.h"
#include "url.h"

char *argv0;

static void ws_cleanup (void);
static BOOL WINAPI ws_handler (DWORD);
static void ws_hangup (void);
static void ws_backgnd (void);
static char *read_registry (HKEY hkey, char *subkey, char *valuename,
			    char *buf, int *len);

/* In log.c.  */
void redirect_output (const char *);

static char *title_buf;

static int isNT;


#if 0 /* no longer used -- see `host.c' */
int
getdomainname (const char *buf, int len)
{
  if (!read_registry (HKEY_LOCAL_MACHINE,
		     "System\\CurrentControlSet\\Services\\Vxd\\MSTCP",
		     "Domain", buf, &len))
    {
      return -1;
    }
  return len;
}
#endif

unsigned int
sleep (unsigned seconds)
{
  Sleep (1000 * seconds);
  /* Unix sleep() is interruptible.  To make it semi-usable, it
     returns a value that says how much it "really" slept, or some
     junk like that.  Ignore it.  */
  return 0U;
}

void
windows_main_junk (int *argc, char **argv, char **exec_name)
{
  char *p;

  argv0 = argv[0];

  /* Remove .EXE from filename if it has one.  */
  *exec_name = xstrdup (*exec_name);
  p = strrchr (*exec_name, '.');
  if (p)
    *p = '\0';

  /* Now we ar going to background if the last argv was `&'.  */
  if (!strcmp (argv[*argc - 1], "&"))
    {
      ws_backgnd ();
      (*argc)--;
    }
}

char *
pwd_cuserid (char *where)
{
  char buf[32], *ptr;
  int len = sizeof (buf);
  if (GetUserName (buf, (LPDWORD) &len) == TRUE)
    {
      ;
    }
  else if (!!(ptr = getenv ("USERNAME")))
    {
      strcpy (buf, ptr);
    }
  else if (!read_registry (HKEY_LOCAL_MACHINE, "Network\\Logon",
			  "username", buf, &len))
    {
      return NULL;
    }
  if (where)
    {
      strncpy (where, buf, len);
      return where;
    }
  return xstrdup (buf);
}

void
ws_startup (void)
{
  WORD requested;
  WSADATA data;
  int err;
  OSVERSIONINFO os;

  if (GetVersionEx (&os) == TRUE
      && os.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS)
    isNT = 1;

  requested = MAKEWORD (1, 1);
  err = WSAStartup (requested, &data);

  if (err != 0)
    {
      fprintf (stderr, _("wget: Couldn't find usable socket driver.\n"));
      exit (1);
    }

  if (LOBYTE (requested) < 1 || (LOBYTE (requested) == 1 &&
				HIBYTE (requested) < 1))
    {
      fprintf (stderr, _("wget: Couldn't find usable socket driver.\n"));
      WSACleanup ();
      exit (1);
    }
  atexit (ws_cleanup);
  SetConsoleCtrlHandler (ws_handler, TRUE);
}

static void
ws_cleanup (void)
{
  WSACleanup ();
}

static void
ws_backgnd (void)
{
  ws_hangup ();
  if (!isNT)
    FreeConsole ();
}

static BOOL WINAPI
ws_handler (DWORD dwEvent)
{
  switch (dwEvent)
    {

#ifdef CTRLC_BACKGND
    case CTRL_C_EVENT:
#endif
#ifdef CTRLBREAK_BACKGND
    case CTRL_BREAK_EVENT:
#endif
      ws_backgnd ();
      break;
    case CTRL_SHUTDOWN_EVENT:
    case CTRL_CLOSE_EVENT:
    case CTRL_LOGOFF_EVENT:
    default:
      WSACleanup ();
      return FALSE;
    }
  return TRUE;
}

static void
ws_hangup (void)
{
  redirect_output (_("\n\
CTRL+Break received, redirecting output to `%s'.\n\
Execution continued in background.\n\
You may stop Wget by pressing CTRL+ALT+DELETE.\n"));
}

void
ws_changetitle (char *url, int nurl)
{
  if (!nurl)
    return;
  title_buf = (char *)xmalloc (strlen (url) + 20);
  sprintf (title_buf, "Wget %s%s", url, nurl == 1 ? "" : " ...");
  SetConsoleTitle (title_buf);
}

static char *
read_registry (HKEY hkey, char *subkey, char *valuename, char *buf, int *len)
{
  HKEY result;
  DWORD size = *len;
  DWORD type = REG_SZ;
  if (RegOpenKeyEx (hkey, subkey, NULL, KEY_READ, &result) != ERROR_SUCCESS)
    return NULL;
  if (RegQueryValueEx (result, valuename, NULL, &type, buf, &size) != ERROR_SUCCESS)
    buf = NULL;
  *len = size;
  RegCloseKey (result);
  return buf;
}

char *
ws_mypath (void)
{
  char *buffer;
  int rrr;
  char *ptr = strrchr (argv0, '\\');
  if (ptr)
    {
      *(ptr + 1) = '\0';
      return argv0;
    }
  buffer = (char*) xmalloc (256);
  if (!!(rrr = SearchPath (NULL, argv0, strchr (argv0, '.') ? NULL : ".EXE",
			  256, buffer, &ptr))
      && rrr <= 256)
    {
      *ptr = '\0';
      return buffer;
    }
  free (buffer);
  return NULL;
}

void
ws_help (const char *name)
{
  char *mypath = ws_mypath ();

  if (mypath)
    {
      char *buf = (char *)alloca (strlen (mypath) + 1 + strlen (name) + 3 + 1);
      sprintf (buf, "%s%s.HLP", mypath, name);
#ifdef DEBUG
      printf (_("Starting WinHelp %s\n"), buf);
#endif
      WinHelp (NULL, buf, HELP_INDEX, NULL);
    }
}
